searchData={"items":[{"type":"module","title":"Electric.Client","doc":"An Elixir client for the [ElectricSQL synchronisation server](https://electric-sql.com/).\n\nSubscribing to a particular\n[Shape](https://electric-sql.com/docs/guides/shapes) produce a stream of\nupdate messages that will allow you to synchronise your local system to the\nstate the Postgres database.","ref":"Electric.Client.html"},{"type":"module","title":"Quickstart - Electric.Client","doc":"","ref":"Electric.Client.html#module-quickstart"},{"type":"module","title":"Start and connect the Electric Sync Service - Electric.Client","doc":"Follow the [quickstart guide](https://electric-sql.com/docs/quickstart) to\nget Electric running and connected to a Postgres database.","ref":"Electric.Client.html#module-start-and-connect-the-electric-sync-service"},{"type":"module","title":"Install the Electric Client and Receive sync events - Electric.Client","doc":"Create a simple script that will subscribe to events from the `foo` table you created as part of the [Quickstart](#quickstart).\n\n``` elixir\n# electric.ex\nMix.install([\n  :electric_client\n])\n\n{:ok, client} = Electric.Client.new(base_url: \"http://localhost:3000\")\n# You can create a stream from a table name or a Shape defined using\n# `ShapeDefinition.new/2`\nstream = Electric.Client.stream(client, \"foo\")\n\nfor msg <- stream do\n  IO.inspect(msg, pretty: true, syntax_colors: IO.ANSI.syntax_colors())\nend\n```\n\nThen run it:\n\n```sh\nelixir electric.ex\n```\n\nIn a separate terminal window, connect to the Postgres database:\n\n```sh\npsql \"postgresql://postgres:password@localhost:54321/electric\"\n```\n\nNow any modifications you make to the data in the `foo` table will appear\nas messages in the elixir process.\n\n```sql\nINSERT INTO foo (name, value) VALUES\n    ('josevalim', 4545),\n    ('eksperimental', 966),\n    ('lexmag', 677),\n    ('whatyouhide', 598),\n    ('ericmj', 583),\n    ('alco', 377);\n\nUPDATE foo SET value = value + 1;\n```","ref":"Electric.Client.html#module-install-the-electric-client-and-receive-sync-events"},{"type":"module","title":"Filtering Using WHERE clauses - Electric.Client","doc":"You can subscribe to subsets of the data in your table using [`where` clauses](https://electric-sql.com/docs/guides/shapes#where-clause).\n\n\n``` elixir\n{:ok, client} = Electric.Client.new(base_url: \"http://localhost:3000\")\n\nshape = Electric.Client.ShapeDefinition.new(\"foo\", where: \"name ILIKE 'a%'\")\n\nstream = Electric.Client.stream(client, shape)\n\nfor msg <- stream do\n  # you will now only receive events for database rows matching the `where` clause\nend\n```","ref":"Electric.Client.html#module-filtering-using-where-clauses"},{"type":"module","title":"Configuration - Electric.Client","doc":"See `new/1` for configuration options of the client itself,\nand `stream/3` for details on configuring the stream itself.","ref":"Electric.Client.html#module-configuration"},{"type":"module","title":"Ecto Integration - Electric.Client","doc":"If you have [Ecto](https://hexdocs.pm/ecto) installed then you can define you Shapes using Ecto queries:\n\n``` elixir\n# ecto.ex\nMix.install([\n  :ecto_sql,\n  :electric_client\n])\n\nimport Ecto.Query, only: [from: 2]\nimport Ecto.Query.API, only: [ilike: 2]\n\ndefmodule Foo do\n  use Ecto.Schema\n\n  schema \"foo\" do\n    field :name, :string\n    field :value, :float\n  end\nend\n\n{:ok, client} = Electric.Client.new(base_url: \"http://localhost:3000\")\n\n# Replace the table or `ShapeDefinition` with an `Ecto` query and set\n# `update_mode` to `:full` to receive full rows for update messages.\n#\n# The normal `update_mode: :modified` setting will only send the changed\n# columns, so we'd end up with partial `%Foo{}` instances.\nstream =\n  Electric.Client.stream(\n    client,\n    from(f in Foo, where: ilike(f.name, \"a%\")),\n    update_mode: :full\n  )\n\nfor %{headers: %{operation: operation}, value: value} <- stream do\n  # The message `value` will now be a `%Foo{}` struct \n  IO.inspect([{operation, value}], pretty: true, syntax_colors: IO.ANSI.syntax_colors())\nend\n```","ref":"Electric.Client.html#module-ecto-integration"},{"type":"module","title":"Custom Values - Electric.Client","doc":"Electric sends all column values as binaries. The `Ecto` integration uses\n`Ecto`'s schema information to turn those into the relevant Elixir terms but\nwe can provide our own `binary() => term()` mapping by implementing the\n[`Electric.Client.ValueMapper` behaviour](`Electric.Client.ValueMapper`).","ref":"Electric.Client.html#module-custom-values"},{"type":"function","title":"Electric.Client.delete_shape/2","doc":"Use the given `client` to delete the `shape` instance from the server.\n\nDelete shape only works if Electric is configured to `allow_shape_deletion`.","ref":"Electric.Client.html#delete_shape/2"},{"type":"function","title":"Electric.Client.new/1","doc":"Create a new client.","ref":"Electric.Client.html#new/1"},{"type":"function","title":"Options - Electric.Client.new/1","doc":"* `:base_url` (`t:String.t/0`) - Required. The URL of the electric server, e.g. for local development this would be `http://localhost:3000`.","ref":"Electric.Client.html#new/1-options"},{"type":"function","title":"Electric.Client.new!/1","doc":"Create a new client with the given options or raise if the configuration is\ninvalid.","ref":"Electric.Client.html#new!/1"},{"type":"function","title":"Electric.Client.shape/2","doc":"A shortcut to `ShapeDefinition.new/2`.","ref":"Electric.Client.html#shape/2"},{"type":"function","title":"Electric.Client.stream/3","doc":"Use the `client` to return a stream of update messages for the given `shape`.\n\n`shape` can be a table name, e.g. `\"my_table\"`, a full `ShapeDefinition`\nincluding a table name and `where` clause, or (if `Ecto` is installed) an\n`Ecto.Queryable` instance, such as an `Ecto.Query` or a `Ecto.Schema` struct.","ref":"Electric.Client.html#stream/3"},{"type":"function","title":"Options - Electric.Client.stream/3","doc":"* `:parser` - A `{module, args}` tuple specifying the `Electric.Client.ValueMapper`\n  implementation to use for mapping values from the sync stream into Elixir\n  terms. The default value is `{Electric.Client.ValueMapper, []}`.\n\n* `:live` (`t:boolean/0`) - If `true` (the default) reads an infinite stream of update messages from the server. The default value is `true`.\n\n* `:update_mode` - Instructs the server to send just the changed columns for an update (`:modified`) or the full row (`:full`). The default value is `:modified`.\n\n* `:oneshot` (`t:boolean/0`) - Only make a single request and then terminate the stream. The default value is `false`.\n\n* `:resume` - Resume the stream from the given point. `Message.ResumeMessage` messages\n  are appended to the change stream if you terminate it early using `live:\n  false` or `oneshot: true`","ref":"Electric.Client.html#stream/3-options"},{"type":"type","title":"Electric.Client.client_option/0","doc":"","ref":"Electric.Client.html#t:client_option/0"},{"type":"type","title":"Electric.Client.client_options/0","doc":"","ref":"Electric.Client.html#t:client_options/0"},{"type":"type","title":"Electric.Client.column/0","doc":"","ref":"Electric.Client.html#t:column/0"},{"type":"type","title":"Electric.Client.cursor/0","doc":"","ref":"Electric.Client.html#t:cursor/0"},{"type":"type","title":"Electric.Client.message/0","doc":"","ref":"Electric.Client.html#t:message/0"},{"type":"type","title":"Electric.Client.param/0","doc":"","ref":"Electric.Client.html#t:param/0"},{"type":"type","title":"Electric.Client.params/0","doc":"","ref":"Electric.Client.html#t:params/0"},{"type":"type","title":"Electric.Client.schema/0","doc":"","ref":"Electric.Client.html#t:schema/0"},{"type":"type","title":"Electric.Client.shape/0","doc":"","ref":"Electric.Client.html#t:shape/0"},{"type":"type","title":"Electric.Client.shape_id/0","doc":"","ref":"Electric.Client.html#t:shape_id/0"},{"type":"type","title":"Electric.Client.shape_option/0","doc":"","ref":"Electric.Client.html#t:shape_option/0"},{"type":"type","title":"Electric.Client.shape_options/0","doc":"","ref":"Electric.Client.html#t:shape_options/0"},{"type":"type","title":"Electric.Client.t/0","doc":"","ref":"Electric.Client.html#t:t/0"},{"type":"type","title":"Electric.Client.table_name/0","doc":"","ref":"Electric.Client.html#t:table_name/0"},{"type":"type","title":"Electric.Client.update_mode/0","doc":"","ref":"Electric.Client.html#t:update_mode/0"},{"type":"behaviour","title":"Electric.Client.Fetch","doc":"","ref":"Electric.Client.Fetch.html"},{"type":"callback","title":"Electric.Client.Fetch.fetch/2","doc":"","ref":"Electric.Client.Fetch.html#c:fetch/2"},{"type":"module","title":"Electric.Client.Fetch.Request","doc":"","ref":"Electric.Client.Fetch.Request.html"},{"type":"type","title":"Electric.Client.Fetch.Request.t/0","doc":"","ref":"Electric.Client.Fetch.Request.html#t:t/0"},{"type":"module","title":"Electric.Client.Fetch.Response","doc":"","ref":"Electric.Client.Fetch.Response.html"},{"type":"type","title":"Electric.Client.Fetch.Response.t/0","doc":"","ref":"Electric.Client.Fetch.Response.html#t:t/0"},{"type":"module","title":"Electric.Client.Message.ChangeMessage","doc":"","ref":"Electric.Client.Message.ChangeMessage.html"},{"type":"function","title":"Electric.Client.Message.ChangeMessage.from_message/2","doc":"","ref":"Electric.Client.Message.ChangeMessage.html#from_message/2"},{"type":"type","title":"Electric.Client.Message.ChangeMessage.key/0","doc":"","ref":"Electric.Client.Message.ChangeMessage.html#t:key/0"},{"type":"type","title":"Electric.Client.Message.ChangeMessage.t/0","doc":"","ref":"Electric.Client.Message.ChangeMessage.html#t:t/0"},{"type":"type","title":"Electric.Client.Message.ChangeMessage.value/0","doc":"","ref":"Electric.Client.Message.ChangeMessage.html#t:value/0"},{"type":"module","title":"Electric.Client.Message.ControlMessage","doc":"","ref":"Electric.Client.Message.ControlMessage.html"},{"type":"function","title":"Electric.Client.Message.ControlMessage.from_message/2","doc":"","ref":"Electric.Client.Message.ControlMessage.html#from_message/2"},{"type":"function","title":"Electric.Client.Message.ControlMessage.must_refetch/0","doc":"","ref":"Electric.Client.Message.ControlMessage.html#must_refetch/0"},{"type":"function","title":"Electric.Client.Message.ControlMessage.up_to_date/0","doc":"","ref":"Electric.Client.Message.ControlMessage.html#up_to_date/0"},{"type":"type","title":"Electric.Client.Message.ControlMessage.control/0","doc":"","ref":"Electric.Client.Message.ControlMessage.html#t:control/0"},{"type":"type","title":"Electric.Client.Message.ControlMessage.t/0","doc":"","ref":"Electric.Client.Message.ControlMessage.html#t:t/0"},{"type":"module","title":"Electric.Client.Message.Headers","doc":"","ref":"Electric.Client.Message.Headers.html"},{"type":"function","title":"Electric.Client.Message.Headers.delete/1","doc":"","ref":"Electric.Client.Message.Headers.html#delete/1"},{"type":"function","title":"Electric.Client.Message.Headers.insert/1","doc":"","ref":"Electric.Client.Message.Headers.html#insert/1"},{"type":"function","title":"Electric.Client.Message.Headers.update/1","doc":"","ref":"Electric.Client.Message.Headers.html#update/1"},{"type":"type","title":"Electric.Client.Message.Headers.operation/0","doc":"","ref":"Electric.Client.Message.Headers.html#t:operation/0"},{"type":"type","title":"Electric.Client.Message.Headers.relation/0","doc":"","ref":"Electric.Client.Message.Headers.html#t:relation/0"},{"type":"type","title":"Electric.Client.Message.Headers.t/0","doc":"","ref":"Electric.Client.Message.Headers.html#t:t/0"},{"type":"module","title":"Electric.Client.Message.ResumeMessage","doc":"Emitted by the synchronisation stream before terminating early. If passed\nas an option to [`Client.stream`](`Electric.Client.stream/3`) allows for\nresuming a shape stream at the given point.\n\nE.g.\n\n```\n# passing `live: false` means the stream will terminate once it receives an\n# `up-to-date` message from the server\nmessages = Electric.Client.stream(client, \"my_table\", live: false) |> Enum.to_list()\n\n%ResumeMessage{} = resume = List.last(messages)\n\n# `stream` will resume from whatever point the initial one finished\nstream = Electric.Client.stream(client, \"my_table\", resume: resume)\n```","ref":"Electric.Client.Message.ResumeMessage.html"},{"type":"type","title":"Electric.Client.Message.ResumeMessage.t/0","doc":"","ref":"Electric.Client.Message.ResumeMessage.html#t:t/0"},{"type":"module","title":"Electric.Client.Mock","doc":"Allows for mocking stream messages.","ref":"Electric.Client.Mock.html"},{"type":"module","title":"Usage - Electric.Client.Mock","doc":"``` elixir\n{:ok, client} = Electric.Client.Mock.new()\n\nusers = [\n  %{id: 1, name: \"User 1\"},\n  %{id: 2, name: \"User 2\"},\n  %{id: 3, name: \"User 3\"}\n]\n\nref = Electric.Client.Mock.async_response(client,\n  status: 200,\n  schema: %{id: %{type: \"int8\"}, name: %{type: \"text\"}},\n  last_offset: Client.Offset.first(),\n  shape_id: \"users-1\",\n  body: Electric.Client.Mock.transaction(users, operation: :insert)\n)\n\nmessages = Electric.Client.stream(client, \"users\", live: false) |> Enum.into([])\n\nrequest = Electric.Client.Mock.async_await(ref, timeout = 5_000)\n```","ref":"Electric.Client.Mock.html#module-usage"},{"type":"function","title":"Electric.Client.Mock.async_await/2","doc":"","ref":"Electric.Client.Mock.html#async_await/2"},{"type":"function","title":"Electric.Client.Mock.async_response/2","doc":"","ref":"Electric.Client.Mock.html#async_response/2"},{"type":"function","title":"Electric.Client.Mock.change/1","doc":"","ref":"Electric.Client.Mock.html#change/1"},{"type":"function","title":"Electric.Client.Mock.headers/1","doc":"","ref":"Electric.Client.Mock.html#headers/1"},{"type":"function","title":"Electric.Client.Mock.new/1","doc":"Create a new mock client, linked to the `parent` process, `self()` by default.","ref":"Electric.Client.Mock.html#new/1"},{"type":"function","title":"Electric.Client.Mock.response/2","doc":"","ref":"Electric.Client.Mock.html#response/2"},{"type":"function","title":"Electric.Client.Mock.transaction/2","doc":"Wrap the given `values` in `Client.Messages.ChangeMessage` structs at the\ngiven `:lsn`.\n\nBy default this will append an `up-to-date` control message to the end of the\nliist of changes. Pass `up_to_date: false` to disable this.","ref":"Electric.Client.Mock.html#transaction/2"},{"type":"function","title":"Electric.Client.Mock.up_to_date/1","doc":"","ref":"Electric.Client.Mock.html#up_to_date/1"},{"type":"type","title":"Electric.Client.Mock.change_opt/0","doc":"","ref":"Electric.Client.Mock.html#t:change_opt/0"},{"type":"type","title":"Electric.Client.Mock.change_opts/0","doc":"","ref":"Electric.Client.Mock.html#t:change_opts/0"},{"type":"type","title":"Electric.Client.Mock.response_opt/0","doc":"","ref":"Electric.Client.Mock.html#t:response_opt/0"},{"type":"type","title":"Electric.Client.Mock.response_opts/0","doc":"","ref":"Electric.Client.Mock.html#t:response_opts/0"},{"type":"type","title":"Electric.Client.Mock.transaction_opt/0","doc":"","ref":"Electric.Client.Mock.html#t:transaction_opt/0"},{"type":"type","title":"Electric.Client.Mock.transaction_opts/0","doc":"","ref":"Electric.Client.Mock.html#t:transaction_opts/0"},{"type":"module","title":"Electric.Client.Offset","doc":"Represents an offset in the synchronisation stream from Electric.","ref":"Electric.Client.Offset.html"},{"type":"function","title":"Electric.Client.Offset.before_all/0","doc":"Return an offset that is guaranteed to be before any real database\noperations.","ref":"Electric.Client.Offset.html#before_all/0"},{"type":"function","title":"Electric.Client.Offset.first/0","doc":"The first possible offset in the log.","ref":"Electric.Client.Offset.html#first/0"},{"type":"function","title":"Electric.Client.Offset.from_string/1","doc":"Parse an offset value from an HTTP header into a `%Elixir.Electric.Client.Offset{}` struct.\n\n    iex> from_string(\"-1\")\n    {:ok, %Elixir.Electric.Client.Offset{tx: -1, op: 0}}\n\n    iex> from_string(\"1378734_3\")\n    {:ok, %Elixir.Electric.Client.Offset{tx: 1378734, op: 3}}\n\n    iex> from_string(\"not a real offset\")\n    {:error, \"has invalid format\"}","ref":"Electric.Client.Offset.html#from_string/1"},{"type":"function","title":"Electric.Client.Offset.from_string!/1","doc":"Parse an offset value from an HTTP header into a `%Elixir.Electric.Client.Offset{}` struct.\n\nRaises if the offset header value is invalid.\n\n    iex> from_string!(\"-1\")\n    %Elixir.Electric.Client.Offset{tx: -1, op: 0}\n\n    iex> from_string!(\"1378734_3\")\n    %Elixir.Electric.Client.Offset{tx: 1378734, op: 3}\n\n    iex> from_string!(\"not a real offset\")\n    ** (ArgumentError) has invalid format","ref":"Electric.Client.Offset.html#from_string!/1"},{"type":"macro","title":"Electric.Client.Offset.is_first/1","doc":"A guard to test if the given offset is the first possible (as returned by `first/0`).","ref":"Electric.Client.Offset.html#is_first/1"},{"type":"function","title":"Electric.Client.Offset.new/2","doc":"Create a new Elixir.Electric.Client.Offset struct from the given LSN and operation\noffsets.\n\n    iex> new(2349, 3)\n    %Elixir.Electric.Client.Offset{tx: 2349, op: 3}","ref":"Electric.Client.Offset.html#new/2"},{"type":"function","title":"Electric.Client.Offset.to_string/1","doc":"Output the an Offset as a string for use in query parameters.\n\n    iex> new(2349, 3) |> Elixir.Electric.Client.Offset.to_string()\n    \"2349_3\"\n\n    iex> before_all() |> Elixir.Electric.Client.Offset.to_string()\n    \"-1\"","ref":"Electric.Client.Offset.html#to_string/1"},{"type":"function","title":"Electric.Client.Offset.to_tuple/1","doc":"","ref":"Electric.Client.Offset.html#to_tuple/1"},{"type":"type","title":"Electric.Client.Offset.int64/0","doc":"","ref":"Electric.Client.Offset.html#t:int64/0"},{"type":"type","title":"Electric.Client.Offset.op_offset/0","doc":"","ref":"Electric.Client.Offset.html#t:op_offset/0"},{"type":"type","title":"Electric.Client.Offset.t/0","doc":"","ref":"Electric.Client.Offset.html#t:t/0"},{"type":"type","title":"Electric.Client.Offset.tx_offset/0","doc":"","ref":"Electric.Client.Offset.html#t:tx_offset/0"},{"type":"module","title":"Electric.Client.ShapeDefinition","doc":"Struct for defining a shape.\n\n    iex> ShapeDefinition.new(\"items\", where: \"something = true\")\n    %ShapeDefinition{table: \"items\", where: \"something = true\"}","ref":"Electric.Client.ShapeDefinition.html"},{"type":"function","title":"Electric.Client.ShapeDefinition.new/2","doc":"Create a `ShapeDefinition` for the given `table_name`.","ref":"Electric.Client.ShapeDefinition.html#new/2"},{"type":"function","title":"Options - Electric.Client.ShapeDefinition.new/2","doc":"* `:where` - Filter the table according to the where clause. The default value is `nil`.\n\n* `:namespace` - The namespace the table belongs to. If `nil` then Postgres will use whatever schema is the default (usually `public`). The default value is `nil`.","ref":"Electric.Client.ShapeDefinition.html#new/2-options"},{"type":"function","title":"Electric.Client.ShapeDefinition.url_table_name/1","doc":"Return a string representation of the shape's table, quoted for use in API URLs.\n\n    iex> ShapeDefinition.url_table_name(ShapeDefinition.new(\"my_table\"))\n    \"my_table\"\n\n    iex> ShapeDefinition.url_table_name(ShapeDefinition.new(\"my_table\", namespace: \"my_app\"))\n    \"my_app.my_table\"\n\n    iex> ShapeDefinition.url_table_name(ShapeDefinition.new(\"my table\", namespace: \"my app\"))\n    \"%22my app%22.%22my table%22\"","ref":"Electric.Client.ShapeDefinition.html#url_table_name/1"},{"type":"type","title":"Electric.Client.ShapeDefinition.option/0","doc":"","ref":"Electric.Client.ShapeDefinition.html#t:option/0"},{"type":"type","title":"Electric.Client.ShapeDefinition.options/0","doc":"","ref":"Electric.Client.ShapeDefinition.html#t:options/0"},{"type":"type","title":"Electric.Client.ShapeDefinition.t/0","doc":"","ref":"Electric.Client.ShapeDefinition.html#t:t/0"},{"type":"behaviour","title":"Electric.Client.ValueMapper","doc":"A behaviour for mapping the value fields of `Message.ChangeMessage` messages\nfrom the shape stream.\n\nThis requires implementing a single function `for_schema/2`.\n\nElectric sends schema information with every response. For example the schema\nof the `foo` table from the [Electric\nQuickstart](https://electric-sql.com/docs/quickstart) is\n\n``` elixir\n%{\n  id: %{type: \"int4\", pk_index: 0, not_null: true},\n  name: %{type: \"varchar\", max_length: 255},\n  value: %{type: \"float8\"}\n}\n```\n\nThe `for_schema/2` function receives this schema as the first argument and it\nmust return a 1-arity function that will map the value structs received from\nElectric to the desired format.\n\nE.g\n\n```elixir\n# the schema for the `foo` table\nschema = %{\n  id: %{type: \"int4\", pk_index: 0, not_null: true},\n  name: %{type: \"varchar\", max_length: 255},\n  value: %{type: \"float8\"}\n}\n\n# get the mapper function for this schema\nmapper_fun = Electric.Client.ValueMapper.for_schema(schema, opts = [])\n\nvalue = %{\n  \"id\" => \"1\",\n  \"name\" => \"James\",\n  \"value\" => \"45.6\"\n}\n\nmapper_fun.(value)\n\n# the mapping parses the integer and float values to their respective\n# Elixir/Erlang types.\n%{\n  \"id\" => 1,\n  \"name\" => \"James\",\n  \"value\" => 45.6\n}\n```\n\nThe current implementation only handles integer and float values. Every other\ncolumn is left as a binary.","ref":"Electric.Client.ValueMapper.html"},{"type":"callback","title":"Electric.Client.ValueMapper.for_schema/2","doc":"Given the schema information passed from the electric server should return a\n1-arity function that takess a values map, which is a map of column names to\nstring values, and returns a version with the values cast to some appropriate\nElixir type.","ref":"Electric.Client.ValueMapper.html#c:for_schema/2"},{"type":"type","title":"Electric.Client.ValueMapper.mapper_fun/0","doc":"","ref":"Electric.Client.ValueMapper.html#t:mapper_fun/0"},{"type":"type","title":"Electric.Client.ValueMapper.opts/0","doc":"","ref":"Electric.Client.ValueMapper.html#t:opts/0"},{"type":"exception","title":"Electric.Client.Error","doc":"","ref":"Electric.Client.Error.html"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}